module Pages.Home_ exposing (Model, Msg, page)

import Config.Data.Identity exposing (pageNames)
import Config.Helpers.Format
    exposing
        ( paragraphFontSize
        , paragraphSpacing
        )
import Config.Helpers.Response
    exposing
        ( pageListCenter
        , topLevelContainer
        )
import Config.Helpers.Viewport exposing (resetViewport)
import Config.Style.Colour exposing (colourTheme)
import Config.Style.Glow exposing (glowDeepDarkGrey)
import Config.Style.Icons.Icons
    exposing
        ( construction2
        , upRootLarge
        , upRootMedium
        , upRootSmall
        )
import Config.Style.Transitions exposing (transitionStyleMedium)
import Effect exposing (Effect)
import Element as E exposing (..)
import Element.Background as B
import Element.Border as D
import Element.Font as F exposing (color)
import Html exposing (Html)
import Html.Attributes as H
import Layouts
import Page exposing (Page)
import Route exposing (Route)
import Shared exposing (..)
import Svg.Attributes as SvgAttr exposing (..)
import View exposing (View)


page : Shared.Model -> Route () -> Page Model Msg
page shared route =
    Page.new
        { init = init
        , update = update
        , subscriptions = subscriptions
        , view = view shared
        }
        |> Page.withLayout toLayout


toLayout : Model -> Layouts.Layout Msg
toLayout model =
    Layouts.Navbar {}



-- INIT


type alias Model =
    {}


init : () -> ( Model, Effect Msg )
init () =
    ( {}
    , Effect.map
        (\_ -> NoOp)
        (Effect.sendCmd resetViewport)
    )



-- UPDATE


type Msg
    = NoOp


update : Msg -> Model -> ( Model, Effect Msg )
update msg model =
    case msg of
        NoOp ->
            ( model
            , Effect.none
            )



-- SUBSCRIPTIONS


subscriptions : Model -> Sub Msg
subscriptions model =
    Sub.none



-- VIEW


view : Shared.Model -> Model -> View Msg
view shared model =
    { title = pageNames.pageHome
    , attributes = []
    , element = homeContainer shared.device
    }


homeContainer : Device -> Element msg
homeContainer device =
    topLevelContainer (homeList device)


homeList : Device -> Element msg
homeList device =
    column pageListCenter
        [ case device.class of
            Desktop ->
                desktopHomePage

            BigDesktop ->
                desktopHomePage

            Tablet ->
                case device.orientation of
                    Portrait ->
                        mobileLargeHomePage

                    Landscape ->
                        mobileLargeHomePage

            Phone ->
                case device.orientation of
                    Portrait ->
                        mobileSmallHomePage

                    Landscape ->
                        mobileSmallHomePage
        ]


homePage : Html msg -> Element msg
homePage image =
    column
        [ centerX
        , centerY
        , E.spacing 20
        ]
        [ el [ centerX, E.width E.fill, E.height <| px 100 ] <|
            html image
        , column
            [ paddingEach
                { top = 15
                , bottom = 15
                , left = 20
                , right = 20
                }
            , glowDeepDarkGrey
            , B.color colourTheme.backgroundDarkGrey
            , D.rounded 10
            , E.width E.fill
            , E.spacing 8
            ]
            [ paragraph
                ([ F.color colourTheme.textLightGrey
                 , paragraphSpacing
                 , paragraphFontSize
                 ]
                    ++ [ centerX ]
                )
                [ text "upRootNutrition is an open source project, created by Nick Hiebert, designed to elevate the quality of nutrition science communication in online discourse. By applying more rigorous systems of reasoning, such as formal logic, upRootNutrition aims to cut through the misinformation and sophistry that are endemic on social media." ]
            ]
        ]


desktopHomePageImage : Html msg
desktopHomePageImage =
    upRootLarge


mobileLargeHomePageImage : Html msg
mobileLargeHomePageImage =
    upRootMedium


mobileSmallHomePageImage : Html msg
mobileSmallHomePageImage =
    upRootSmall


desktopHomePage =
    homePage desktopHomePageImage


mobileLargeHomePage =
    homePage mobileLargeHomePageImage


mobileSmallHomePage =
    homePage mobileSmallHomePageImage
